import 'dart:async';

import 'package:awesome_notifications/awesome_notifications.dart';
import 'package:code_id_flutter/code_services/alice/core/alice_utils.dart';
import 'package:code_id_flutter/code_services/alice/helper/alice_save_helper.dart';
import 'package:code_id_flutter/code_services/alice/model/alice_http_call.dart';
import 'package:code_id_flutter/code_services/alice/model/alice_http_error.dart';
import 'package:code_id_flutter/code_services/alice/model/alice_http_response.dart';
import 'package:code_id_flutter/code_services/alice/ui/page/alice_calls_list_screen.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/material.dart';
// import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:rxdart/rxdart.dart';

/// Streams are created so that app can respond to notification-related events
/// since the plugin is initialised in the `main` function

class ReceivedNotification {
  ReceivedNotification({
    required this.id,
    required this.title,
    required this.body,
    required this.payload,
  });

  final int id;
  final String? title;
  final String? body;
  final String? payload;
}

class AliceCore {
  /// Should user be notified with notification if there's new request catched
  /// by Alice
  final bool showNotification;

  /// Should inspector use dark theme
  final bool darkTheme;

  /// Rx subject which contains all intercepted http calls
  final BehaviorSubject<IList<AliceHttpCall>> callsSubject =
      BehaviorSubject.seeded(<AliceHttpCall>[].lock);

  /// Icon url for notification
  final String? notificationIcon;

  ///Max number of calls that are stored in memory. When count is reached, FIFO
  ///method queue will be used to remove elements.
  final int maxCallsCount;

  ///Directionality of app. If null then directionality of context will be used.
  final TextDirection? directionality;

  // late FlutterLocalNotificationsPlugin _flutterLocalNotificationsPlugin;
  late AwesomeNotifications _awesomeNotifications;

  GlobalKey<NavigatorState>? navigatorKey;
  Brightness _brightness = Brightness.light;
  bool _isInspectorOpened = false;
  StreamSubscription<IList<AliceHttpCall>>? _callsSubscription;
  String? _notificationMessage;
  String? _notificationMessageShown;
  bool _notificationProcessing = false;

  /// Creates alice core instance
  AliceCore(
    this.navigatorKey, {
    required this.showNotification,
    required this.darkTheme,
    this.notificationIcon,
    required this.maxCallsCount,
    this.directionality,
  }) {
    if (showNotification) {
      debugPrint('SHOWW NOTIFICATIONS');
      _initializeNotificationsPlugin();
      _callsSubscription = callsSubject.listen((_) => _onCallsChanged());
    }
    _brightness = darkTheme ? Brightness.dark : Brightness.light;
  }

  /// Dispose subjects and subscriptions
  void dispose() {
    callsSubject.close();
    _awesomeNotifications.actionSink.close();
    _callsSubscription?.cancel();
  }

  /// Get currently used brightness
  Brightness get brightness => _brightness;

  // void _initializeNotificationsPlugin() {
  //   debugPrint('INITIALIZA BEGIN');
  //   _flutterLocalNotificationsPlugin = FlutterLocalNotificationsPlugin();
  //   final initializationSettingsAndroid =
  //       AndroidInitializationSettings(notificationIcon);
  //   final IOSInitializationSettings initializationSettingsIOS =
  //       IOSInitializationSettings();

  //   final initializationSettings = InitializationSettings(
  //       android: initializationSettingsAndroid, iOS: initializationSettingsIOS);
  //   _flutterLocalNotificationsPlugin.initialize(initializationSettings,
  //       onSelectNotification: _onSelectedNotification);
  // }

  void _initializeNotificationsPlugin() {
    debugPrint('INITIALIZA BEGIN');
    _awesomeNotifications = AwesomeNotifications();

    String channelId = "Alice";
    String channelName = "Alice";
    String channelDescription = "Alice";
    _awesomeNotifications.initialize('resource://$notificationIcon', [
      NotificationChannel(
        channelKey: channelId,
        channelName: channelName,
        channelDescription: channelDescription,
        importance: NotificationImportance.High,
        playSound: false,
      )
    ]);
    _awesomeNotifications.isNotificationAllowed().then((isAllowed) {
      if (!isAllowed) {
        // Insert here your friendly dialog box before call the request method
        // This is very important to not harm the user experience
        _awesomeNotifications.requestPermissionToSendNotifications();
      }
    });
    _awesomeNotifications.actionStream.listen((event) {
      _onSelectedNotification(event.payload);
    });
  }

  void _onCallsChanged() async {
    if (callsSubject.value.isNotEmpty) {
      debugPrint('ON CALL CHANGED');
      _notificationMessage = _getNotificationMessage();
      if (_notificationMessage == null ||
          (_notificationMessage != _notificationMessageShown &&
              !_notificationProcessing)) {
        await _showLocalNotification();
        _onCallsChanged();
      }
    }
  }

  Future<void> _onSelectedNotification(Map<String, dynamic>? payload) async {
    debugPrint("CHECK PAYLOAD");
    if (payload != null) {
      debugPrint('PAYLOAD $payload');
      navigateToCallListScreen();
    }
    return;
  }

  /// Opens Http calls inspector. This will navigate user to the new fullscreen
  /// page where all listened http calls can be viewed.
  void navigateToCallListScreen() {
    final context = getContext();
    if (context == null) {
      AliceUtils.log(
          "Cant start Alice HTTP Inspector. Please add NavigatorKey to your application");
      return;
    }
    if (!_isInspectorOpened) {
      _isInspectorOpened = true;
      Navigator.push<void>(
        context,
        MaterialPageRoute(
          builder: (context) => AliceCallsListScreen(this),
        ),
      ).then((onValue) => _isInspectorOpened = false);
    }
  }

  /// Get context from navigator key. Used to open inspector route.
  BuildContext? getContext() => navigatorKey?.currentState?.overlay?.context;

  String _getNotificationMessage() {
    final IList<AliceHttpCall> calls = callsSubject.value;
    final int successCalls = calls
        .where((call) =>
            call.response != null &&
            call.response!.status! >= 200 &&
            call.response!.status! < 300)
        .toList()
        .length;

    debugPrint('====================Success Call ===================');
    debugPrint('$successCalls');

    final int redirectCalls = calls
        .where((call) =>
            call.response != null &&
            call.response!.status! >= 300 &&
            call.response!.status! < 400)
        .toList()
        .length;

    debugPrint('====================Redirect Call ===================');
    debugPrint('$redirectCalls');

    final int errorCalls = calls
        .where((call) =>
            call.response != null &&
            call.response!.status! >= 400 &&
            call.response!.status! < 600)
        .toList()
        .length;
    debugPrint('====================Error Call ===================');
    debugPrint('$errorCalls');

    final int loadingCalls =
        calls.where((call) => call.loading).toList().length;
    debugPrint('====================Loadings Call ===================');
    debugPrint('$loadingCalls');

    final StringBuffer notificationsMessage = StringBuffer();
    if (loadingCalls > 0) {
      notificationsMessage.write("Loading: $loadingCalls");
      notificationsMessage.write(" | ");
    }
    if (successCalls > 0) {
      notificationsMessage.write("Success: $successCalls");
      notificationsMessage.write(" | ");
    }
    if (redirectCalls > 0) {
      notificationsMessage.write("Redirect: $redirectCalls");
      notificationsMessage.write(" | ");
    }
    if (errorCalls > 0) {
      notificationsMessage.write("Error: $errorCalls");
    }
    String notificationMessageString = notificationsMessage.toString();
    debugPrint('==================== Notification Message ===================');
    debugPrint(notificationMessageString);
    if (notificationMessageString.endsWith(" | ")) {
      notificationMessageString = notificationMessageString.substring(
          0, notificationMessageString.length - 3);
    }

    return notificationMessageString;
  }

  // Future _showLocalNotification() async {
  //   _notificationProcessing = true;
  //   String channelId = "Alice";
  //   String channelName = "Alice";
  //   String channelDescription = "Alice";
  //   final androidPlatformChannelSpecifics = AndroidNotificationDetails(
  //       channelId, channelName,
  //       channelDescription: channelDescription,
  //       enableVibration: false,
  //       playSound: false,
  //       largeIcon: DrawableResourceAndroidBitmap(notificationIcon));
  //   final IOSNotificationDetails iOSPlatformChannelSpecifics =
  //       IOSNotificationDetails(
  //           presentAlert: true,
  //           presentBadge: true,
  //           presentSound: false,
  //           threadIdentifier: 'thread_id');
  //   final NotificationDetails platformChannelSpecifics = NotificationDetails(
  //       android: androidPlatformChannelSpecifics,
  //       iOS: iOSPlatformChannelSpecifics);
  //   final String? message = _notificationMessage;
  //   final int totalRequest = callsSubject.value.length;
  //   debugPrint(
  //       '=========================== Show Local Notification =======================');
  //   debugPrint('Total Request $totalRequest');
  //   debugPrint('Message $message');

  //   await _flutterLocalNotificationsPlugin.show(
  //       0,
  //       "Alice (total: $totalRequest requests)",
  //       message,
  //       platformChannelSpecifics,
  //       payload: "Alice");
  //   _notificationMessageShown = message;
  //   _notificationProcessing = false;
  //   return;
  // }

  Future _showLocalNotification() async {
    _notificationProcessing = true;
    String channelId = "Alice";
    String channelName = "Alice";
    String channelDescription = "Alice";
    final String? message = _notificationMessage;
    final int totalRequest = callsSubject.value.length;
    await _awesomeNotifications.createNotification(
        content: NotificationContent(
            id: 0,
            channelKey: channelId,
            title: "Alice (total: $totalRequest requests)",
            body: message,
            displayOnBackground: true,
            displayOnForeground: true,
            createdSource: NotificationSource.Local,
            // notificationLayout: ,
            payload: {"alice": "alice"}));

    debugPrint(
        '=========================== Show Local Notification =======================');
    debugPrint('Total Request $totalRequest');
    debugPrint('Message $message');
    _notificationMessageShown = message;
    _notificationProcessing = false;
    return;
  }

  /// Add alice http call to calls subject
  void addCall(AliceHttpCall call) {
    final callsCount = callsSubject.value.length;
    if (callsCount >= maxCallsCount) {
      IList<AliceHttpCall> originalCalls = callsSubject.value;
      IList<AliceHttpCall> calls =
          List<AliceHttpCall>.from(originalCalls).toIList();
      calls.sort(
          (call1, call2) => call1.createdTime.compareTo(call2.createdTime));
      final indexToReplace = originalCalls.indexOf(calls.first);
      originalCalls = originalCalls.replace(indexToReplace, call);

      callsSubject.add(originalCalls);
    } else {
      callsSubject.add([...callsSubject.value, call].lock);
    }
  }

  /// Add error to existing alice http call
  void addError(AliceHttpError error, int requestId) {
    final AliceHttpCall? selectedCall = _selectCall(requestId);

    if (selectedCall == null) {
      debugPrint("Selected call is null");
      return;
    }

    selectedCall.error = error;
    callsSubject.add([...callsSubject.value].lock);
  }

  /// Add response to existing alice http call
  void addResponse(AliceHttpResponse response, int requestId) {
    final AliceHttpCall? selectedCall = _selectCall(requestId);

    if (selectedCall == null) {
      debugPrint("Selected call is null");
      return;
    }
    selectedCall.loading = false;
    selectedCall.response = response;
    selectedCall.duration = response.time.millisecondsSinceEpoch -
        selectedCall.request!.time.millisecondsSinceEpoch;

    callsSubject.add([...callsSubject.value].lock);
  }

  /// Add alice http call to calls subject
  void addHttpCall(AliceHttpCall aliceHttpCall) {
    // assert(aliceHttpCall.request != null, "Http call request can't be null");
    // assert(aliceHttpCall.response != null, "Http call response can't be null");
    callsSubject.add([...callsSubject.value, aliceHttpCall].lock);
  }

  /// Remove all calls from calls subject
  void removeCalls() {
    callsSubject.add(<AliceHttpCall>[].lock);
  }

  AliceHttpCall? _selectCall(int requestId) =>
      callsSubject.value.firstWhere((call) => call.id == requestId);

  /// Save all calls to file
  void saveHttpRequests(BuildContext context) {
    AliceSaveHelper.saveCalls(context, callsSubject.value, _brightness);
  }
}
